#!/usr/bin/env python3
"""
Rhinovirus Data Retrieval Module for Report 004

This module retrieves rhinovirus metagenomic data from LabKey using executeSQL API.
It queries the metagenomics_hits_top_hit table for records where 'rhinovirus' 
(case insensitive) appears in the rank field.

Usage as a module:
    from retrieve_rhinovirus_data import retrieve_rhinovirus_data
    
    # Get all rhinovirus data
    df = retrieve_rhinovirus_data()
"""

import sys
import os
from pathlib import Path
from typing import Dict, List, Any, Optional
import pandas as pd
from rich.console import Console
from rich.progress import Progress, SpinnerColumn, TextColumn, TimeElapsedColumn
import pickle
import gzip
from datetime import datetime

# Load environment variables from .env file if running locally
try:
    from dotenv import load_dotenv
    # Look for .env file in the project root
    env_path = Path(__file__).parent.parent.parent / '.env'
    if env_path.exists():
        load_dotenv(env_path)
except ImportError:
    # dotenv not available, rely on environment variables being set externally
    pass

# Add the labkey_api module to the path
sys.path.insert(0, str(Path(__file__).parent.parent.parent / 'labkey_api'))

from utils.labkey_client import LabKeyClient


def get_rhinovirus_query() -> str:
    """
    Generate SQL query to retrieve rhinovirus records from metagenomic_hits_top_hit_etl.
    
    Returns:
        SQL query string
    """
    query = """
    SELECT 
        sewershed_location,
        sample_date,
        rank,
        species,
        stitle,
        mapped_reads,
        total_reads
    FROM metagenomic_hits_top_hit_etl
    WHERE LOWER(rank) LIKE '%rhinovirus%'
      AND sewershed_location IS NOT NULL
      AND sewershed_location <> ''
      AND sample_date IS NOT NULL
    ORDER BY sample_date DESC, sewershed_location
    """
    return query


def retrieve_rhinovirus_data(force_refresh: bool = False) -> pd.DataFrame:
    """
    Retrieve rhinovirus data from LabKey.
    
    Args:
        force_refresh: Force a fresh query even if cached data exists
        
    Returns:
        DataFrame containing rhinovirus metagenomic data
    """
    console = Console()
    
    # Check for cached data first
    cache_dir = Path(__file__).parent / '.cache'
    cache_file = cache_dir / 'rhinovirus_data.pkl.gz'
    
    if not force_refresh and cache_file.exists():
        try:
            with gzip.open(cache_file, 'rb') as f:
                cache_data = pickle.load(f)
            
            # Check cache age (warn if older than 7 days)
            cache_age = datetime.now() - cache_data['timestamp']
            if cache_age.days > 7:
                console.print(f"[yellow]⚠️  Cache is {cache_age.days} days old. Consider refreshing with --force-refresh[/yellow]")
            
            df = cache_data['data']
            console.print(f"[green]✓ Loaded {len(df)} rhinovirus records from cache[/green]")
            console.print(f"  Cache age: {cache_age.days} days")
            return df
        except Exception as e:
            console.print(f"[yellow]⚠️  Failed to load cache: {e}[/yellow]")
    
    # Initialize LabKey client
    try:
        client = LabKeyClient()
    except Exception as e:
        console.print(f"[red]✗ Failed to initialize LabKey client: {e}[/red]")
        raise
    
    # Get the query
    query = get_rhinovirus_query()
    
    # Execute query with progress indicator
    with Progress(
        SpinnerColumn(),
        TextColumn("[progress.description]{task.description}"),
        TimeElapsedColumn(),
        console=console,
    ) as progress:
        task = progress.add_task("Querying rhinovirus data from LabKey...", total=None)
        
        try:
            # Execute SQL query - metagenomics_hits_top_hit is in the 'lists' schema
            result = client.execute_sql('lists', query)
            df = pd.DataFrame(result['rows'])
            progress.update(task, completed=100)
        except Exception as e:
            console.print(f"[red]✗ Query failed: {e}[/red]")
            raise
    
    # Convert sample_date to datetime
    if 'sample_date' in df.columns:
        df['sample_date'] = pd.to_datetime(df['sample_date'])
    
    # Sort by date and location
    df = df.sort_values(['sample_date', 'sewershed_location'], ascending=[False, True])
    
    console.print(f"[green]✓ Retrieved {len(df)} rhinovirus records[/green]")
    
    # Display summary statistics
    if len(df) > 0:
        unique_locations = df['sewershed_location'].nunique()
        date_range = f"{df['sample_date'].min().strftime('%Y-%m-%d')} to {df['sample_date'].max().strftime('%Y-%m-%d')}"
        
        console.print(f"  Locations: {unique_locations}")
        console.print(f"  Date range: {date_range}")
        
        # Show rhinovirus type distribution
        console.print("\n[bold]Rhinovirus types found:[/bold]")
        rank_counts = df['rank'].value_counts().head(10)
        for rank, count in rank_counts.items():
            console.print(f"  • {rank}: {count} records")
    
    # Cache the results
    cache_dir.mkdir(exist_ok=True)
    cache_data = {
        'timestamp': datetime.now(),
        'data': df
    }
    with gzip.open(cache_file, 'wb') as f:
        pickle.dump(cache_data, f)
    console.print(f"[dim]💾 Cached results to {cache_file}[/dim]")
    
    return df


def save_to_cache(df: pd.DataFrame, cache_file: Path) -> None:
    """Save DataFrame to compressed cache file."""
    cache_data = {
        'timestamp': datetime.now(),
        'data': df
    }
    cache_file.parent.mkdir(exist_ok=True)
    with gzip.open(cache_file, 'wb') as f:
        pickle.dump(cache_data, f)


def load_from_cache(cache_file: Path) -> Optional[Dict[str, Any]]:
    """Load DataFrame from compressed cache file."""
    if cache_file.exists():
        try:
            with gzip.open(cache_file, 'rb') as f:
                return pickle.load(f)
        except Exception:
            return None
    return None


if __name__ == "__main__":
    # Allow running directly for testing
    import argparse
    
    parser = argparse.ArgumentParser(description="Retrieve rhinovirus data from LabKey")
    parser.add_argument('--force-refresh', action='store_true', 
                        help='Force refresh data from LabKey (ignore cache)')
    parser.add_argument('--export-csv', type=str, 
                        help='Export results to CSV file')
    
    args = parser.parse_args()
    
    # Retrieve the data
    df = retrieve_rhinovirus_data(force_refresh=args.force_refresh)
    
    # Export to CSV if requested
    if args.export_csv:
        df.to_csv(args.export_csv, index=False)
        console = Console()
        console.print(f"[green]✓ Exported {len(df)} records to {args.export_csv}[/green]")